import ctypes
from .__library__ import lib, get_error
from .data import *


class Point2d():
    """
    2D point for indexing into array
    """
    def __del__(self):
        if self._ptr:
            lib.tof_point_2d_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr = None):
        """
        Construct new point_2d class
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def row(self):
        """
        Get the row value

        Returns
        ----------
        int
            Row value
        """
        return lib.tof_point_2d_row(self._ptr, get_error())


    def col(self):
        """
        Get the column value

        Returns
        ----------
        int
            Column value
        """
        return lib.tof_point_2d_col(self._ptr, get_error())


class Calibration():
    """
    This class contains all the calibration information
    """
    def __del__(self):
        if self._ptr:
            lib.tof_calibration_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, file_location = None, ptr = None):
        """
        Construct new calibration class
        ----------

        Create default calibration
        ----------

        Read calibration from disk

        Parameters
        ----------
        file_location : str
                Location of calibration file
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif file_location != None:
            self._ptr = lib.tof_calibration_new_from_disk(file_location.encode("utf-8"), get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def write(self, file_location):
        """
        Write calibration to disk

        Parameters
        ----------
        file_location : str
            Location to save the calibration file to
        """
        return lib.tof_calibration_write(self._ptr, file_location.encode("utf-8"), get_error())


    def getCalibratedFrequencies(self):
        """
        The the calibrated frequencies in the calibration file

        Returns
        ----------
        List[float]
            The calibrated modulation frequencies
        """
        size = lib.tof_calibration_get_calibrated_frequencies(self._ptr, ctypes.c_float(), 0, get_error())
        array = (ctypes.c_float * size)()
        lib.tof_calibration_get_calibrated_frequencies(self._ptr, array, size, get_error())
        return array[:]


    def getFocalLengthX(self):
        """
        Get focal length x

        Returns
        ----------
        float
            Focal length
        """
        return lib.tof_calibration_get_focal_length_x(self._ptr, get_error())


    def getFocalLengthY(self):
        """
        Get focal length y

        Returns
        ----------
        float
            Focal length
        """
        return lib.tof_calibration_get_focal_length_y(self._ptr, get_error())


    def getPrincipalPointX(self):
        """
        Get principal point x

        Returns
        ----------
        float
            Principal point
        """
        return lib.tof_calibration_get_principal_point_x(self._ptr, get_error())


    def getPrincipalPointY(self):
        """
        Get principal point y

        Returns
        ----------
        float
            Principal point
        """
        return lib.tof_calibration_get_principal_point_y(self._ptr, get_error())


    def getDepthCameraMatrix(self):
        """
        Get the camera matrix of the depth sensor and lens

        Returns
        ----------
        List[float]
            Camera matrix, 3x3 matrix
        """
        array = lib.tof_calibration_get_depth_camera_matrix(self._ptr, get_error())
        return array.contents[:]


    def getDepthDistortionCoefficients(self):
        """
        Get the distortion coefficients of the depth lens

        Returns
        ----------
        List[float]
            Distortion coefficients, 5x1 matrix
        """
        array = lib.tof_calibration_get_depth_distortion_coefficients(self._ptr, get_error())
        return array.contents[:]


    def getRgbCameraMatrix(self):
        """
        Get the camera matrix of the rgb sensor and lens

        Returns
        ----------
        List[float]
            Camera matrix, 3x3 matrix
        """
        array = lib.tof_calibration_get_rgb_camera_matrix(self._ptr, get_error())
        return array.contents[:]


    def getRgbDistortionCoefficients(self):
        """
        Get the distortion coefficients of the rgb lens

        Returns
        ----------
        List[float]
            Distortion coefficients, 5x1 matrix
        """
        array = lib.tof_calibration_get_rgb_distortion_coefficients(self._ptr, get_error())
        return array.contents[:]


    def getRotationVector(self):
        """
        Get the rotation vector between the depth and rgb sensor

        Returns
        ----------
        List[float]
            Rotation vector, 3x1 matrix
        """
        array = lib.tof_calibration_get_rotation_vector(self._ptr, get_error())
        return array.contents[:]


    def getTranslationVector(self):
        """
        Get the translation vector between the depth and rgb sensor

        Returns
        ----------
        List[float]
            Translation vector, 3x1 matrix
        """
        array = lib.tof_calibration_get_translation_vector(self._ptr, get_error())
        return array.contents[:]


def transform3dToDepth2d(x, y, z, calibration):
    """
    Transform a 3D point into a 2D pixel coordinate of the depth camera

    Parameters
    ----------
    x : float
        X value
    y : float
        Y value
    z : float
        Z value
    calibration : Calibration
        The camera calibration

    Returns
    ----------
    Point2d
        2D pixel coordinate of the depth camera
    """
    return Point2d(ptr = lib.tof_transform_3d_to_depth_2d(x, y, z, calibration._ptr, get_error()))


def transform3dToColor2d(x, y, z, calibration):
    """
    Transform a 3D point into a 2D pixel coordinate of the color camera

    Parameters
    ----------
    x : float
        X value
    y : float
        Y value
    z : float
        Z value
    calibration : Calibration
        The camera calibration

    Returns
    ----------
    Point2d
        2D pixel coordinate of the color camera
    """
    return Point2d(ptr = lib.tof_transform_3d_to_color_2d(x, y, z, calibration._ptr, get_error()))


def transformColor2dToDepth2d(row, col, depth_data, calibration):
    """
    Transform a 2D pixel coordinate of the color camera to the 2D pixel coordinate
    of the depth camera.

    Parameters
    ----------
    row : int
        Row coordinate
    col : int
        Column cooridnate
    depth_data : Data
        Depth data containing xyz information, XYZ, XYZAmp, XYZBGR and XYZBGRI are
        all valid
    calibration : Calibration
        The camera calibration

    Returns
    ----------
    Point2d
        2D pixel coordinate of the depth camera
    """
    return Point2d(ptr = lib.tof_transform_color_2d_to_depth_2d(row, col, depth_data._ptr, calibration._ptr, get_error()))


